#!/usr/bin/env python

from setuptools import setup

if __name__ == '__main__':
    setup(
          name = 'shtub',
          version = '0.2.9',
          description = 'shtub - shell command stub',
          long_description = '''''',
          author = "Alexander Metzner, Michael Gruber, Udo Juettner",
          author_email = "alexander.metzner@gmail.com, aelgru@gmail.com, udo.juettner@gmail.com",
          license = 'GNU GPL v3',
          url = 'https://github.com/yadt/shtub',
          scripts = [],
          packages = ['shtub'],
          classifiers = ['Development Status :: 3 - Alpha', 'Programming Language :: Python'],
          
          
          
          
          zip_safe=True
    )
